package org.expeditee.auth;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.KeyStore;
import java.security.KeyStore.SecretKeyEntry;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableEntryException;
import java.security.cert.CertificateException;

import javax.crypto.SecretKey;

import org.expeditee.gui.FrameIO;
import org.ngikm.cryptography.CryptographyConstants;

public final class Authenticator implements CryptographyConstants {
	final KeyStore keyStore = KeyStore.getInstance(KeystoreType);
	private static final String keystoreFileName = FrameIO.PARENT_FOLDER + "keystore.ks" + File.separator;
	
	public Authenticator() throws KeyStoreException, FileNotFoundException, IOException, NoSuchAlgorithmException, CertificateException {
		final File keyStoreFile = new File(keystoreFileName);
		if (!keyStoreFile.exists()) { 
			keyStore.load(null, "ExpediteeAuthPassword".toCharArray());
		} else {
			try (final InputStream in = new FileInputStream(keystoreFileName)) {
				keyStore.load(in, "ExpediteeAuthPassword".toCharArray());
			}
		}
	}
	
	public final SecretKey getSecretKey(final String label, final String password) throws NoSuchAlgorithmException, UnrecoverableEntryException, KeyStoreException {		
		final KeyStore.ProtectionParameter entryPassword = new KeyStore.PasswordProtection(password.toCharArray());
		final KeyStore.SecretKeyEntry entry = (SecretKeyEntry) keyStore.getEntry(label, entryPassword);
		if (entry == null) { return null; }
		else { return entry.getSecretKey(); }
	}
	
	public final void putKey(final String label, final String password, final SecretKey key) throws KeyStoreException, NoSuchAlgorithmException, CertificateException, FileNotFoundException, IOException {
		final KeyStore.SecretKeyEntry entry = new KeyStore.SecretKeyEntry(key);
		final KeyStore.ProtectionParameter entryPassword = new KeyStore.PasswordProtection(password.toCharArray());
		keyStore.setEntry(label, entry, entryPassword);
		keyStore.store(new FileOutputStream(keystoreFileName), "ExpediteeAuthPassword".toCharArray());
	}
}
